package cn.com.infosec.crypto.generators;

import cn.com.infosec.crypto.AsymmetricCipherKeyPair;
import cn.com.infosec.crypto.AsymmetricCipherKeyPairGenerator;
import cn.com.infosec.crypto.KeyGenerationParameters;
import cn.com.infosec.crypto.params.NaccacheSternKeyGenerationParameters;
import cn.com.infosec.crypto.params.NaccacheSternKeyParameters;
import cn.com.infosec.crypto.params.NaccacheSternPrivateKeyParameters;
import java.io.PrintStream;
import java.math.BigInteger;
import java.security.SecureRandom;
import java.util.Vector;

public class NaccacheSternKeyPairGenerator implements AsymmetricCipherKeyPairGenerator {
   private static int[] smallPrimes = new int[]{3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59, 61, 67, 71, 73, 79, 83, 89, 97, 101, 103, 107, 109, 113, 127, 131, 137, 139, 149, 151, 157, 163, 167, 173, 179, 181, 191, 193, 197, 199, 211, 223, 227, 229, 233, 239, 241, 251, 257, 263, 269, 271, 277, 281, 283, 293, 307, 311, 313, 317, 331, 337, 347, 349, 353, 359, 367, 373, 379, 383, 389, 397, 401, 409, 419, 421, 431, 433, 439, 443, 449, 457, 461, 463, 467, 479, 487, 491, 499, 503, 509, 521, 523, 541, 547, 557};
   private static final BigInteger ONE = BigInteger.valueOf(1L);
   private NaccacheSternKeyGenerationParameters param;

   private static BigInteger generatePrime(int bitLength, int certainty, SecureRandom rand) {
      BigInteger var3;
      var3 = new BigInteger.<init>(bitLength, certainty, rand);

      while(var3.bitLength() != bitLength) {
         var3 = new BigInteger.<init>(bitLength, certainty, rand);
      }

      return var3;
   }

   private static Vector permuteList(Vector arr, SecureRandom rand) {
      Vector var2;
      var2 = new Vector.<init>();
      Vector var3;
      var3 = new Vector.<init>();

      for(int var4 = 0; var4 < arr.size(); ++var4) {
         var3.addElement(arr.elementAt(var4));
      }

      var2.addElement(var3.elementAt(0));
      var3.removeElementAt(0);

      while(var3.size() != 0) {
         Object arr1 = var3.elementAt(0);
         var2.insertElementAt(arr1, getInt(rand, var2.size() + 1));
         var3.removeElementAt(0);
      }

      return var2;
   }

   private static int getInt(SecureRandom rand, int n) {
      if ((n & -n) == n) {
         return (int)((long)n * (long)(rand.nextInt() & Integer.MAX_VALUE) >> 31);
      } else {
         int var10000;
         int var2;
         do {
            var10000 = rand.nextInt() & Integer.MAX_VALUE;
         } while(var10000 - (var2 = var10000 % n) + (n - 1) < 0);

         return var2;
      }
   }

   private static Vector findFirstPrimes(int count) {
      Vector var1;
      var1 = new Vector.<init>(count);

      for(int var2 = 0; var2 != count; ++var2) {
         var1.addElement(BigInteger.valueOf((long)smallPrimes[var2]));
      }

      return var1;
   }

   public void init(KeyGenerationParameters param) {
      this.param = (NaccacheSternKeyGenerationParameters)param;
   }

   public AsymmetricCipherKeyPair generateKeyPair() {
      int var1 = this.param.getStrength();
      SecureRandom var2 = this.param.getRandom();
      int var3 = this.param.getCertainty();
      boolean var4;
      if (var4 = this.param.isDebug()) {
         System.out.println("Fetching first " + this.param.getCntSmallPrimes() + " primes.");
      }

      Vector var20 = permuteList(findFirstPrimes(this.param.getCntSmallPrimes()), var2);
      BigInteger var5 = ONE;
      int var6 = 0;

      BigInteger var7;
      for(var7 = var5; var6 < var20.size() / 2; ++var6) {
         var7 = var7.multiply((BigInteger)var20.elementAt(var6));
      }

      for(var6 = var20.size() / 2; var6 < var20.size(); ++var6) {
         var5 = var5.multiply((BigInteger)var20.elementAt(var6));
      }

      BigInteger var8;
      int var10001 = (var1 - (var8 = var7.multiply(var5)).bitLength() - 48) / 2 + 1;
      BigInteger var9 = generatePrime(var10001, var3, var2);
      BigInteger var10 = generatePrime(var10001, var3, var2);
      long var11 = 0L;
      if (var4) {
         System.out.println("generating p and q");
      }

      BigInteger var13 = var9.multiply(var7).shiftLeft(1);
      BigInteger var14 = var10.multiply(var5).shiftLeft(1);

      while(true) {
         while(true) {
            ++var11;
            BigInteger var15;
            BigInteger var16;
            if ((var16 = (var15 = generatePrime(24, var3, var2)).multiply(var13).add(ONE)).isProbablePrime(var3)) {
               BigInteger var25;
               while(var15.equals(var5 = generatePrime(24, var3, var2)) || !(var7 = var5.multiply(var14).add(var25 = ONE)).isProbablePrime(var3)) {
               }

               if (var8.gcd(var15.multiply(var5)).equals(var25)) {
                  if (var16.multiply(var7).bitLength() >= var1) {
                     PrintStream var10000;
                     StringBuilder var30;
                     if (var4) {
                        var10000 = System.out;
                        StringBuilder var26;
                        var30 = var26 = new StringBuilder;
                        var26.<init>("needed ");
                        var10000.println(var30.append(var11).append(" tries to generate p and q.").toString());
                     }

                     BigInteger var17 = var16.multiply(var7);
                     var25 = var16.subtract(var25).multiply(var7.subtract(var25));
                     var11 = 0L;
                     if (var4) {
                        System.out.println("generating g");
                     }

                     while(true) {
                        Vector var18;
                        var18 = new Vector.<init>();

                        for(int var27 = 0; var27 != var20.size(); ++var27) {
                           var14 = var25.divide((BigInteger)var20.elementAt(var27));

                           BigInteger var19;
                           BigInteger var29;
                           do {
                              ++var11;
                              var29 = var19 = new BigInteger;
                              var19.<init>(var1, var3, var2);
                           } while(var29.modPow(var14, var17).equals(ONE));

                           var18.addElement(var19);
                        }

                        var13 = ONE;

                        int var28;
                        for(var28 = 0; var28 < var20.size(); ++var28) {
                           var13 = var13.multiply(((BigInteger)var18.elementAt(var28)).modPow(var8.divide((BigInteger)var20.elementAt(var28)), var17)).mod(var17);
                        }

                        boolean var33 = false;

                        for(var28 = 0; var28 < var20.size(); ++var28) {
                           if (var13.modPow(var25.divide((BigInteger)var20.elementAt(var28)), var17).equals(ONE)) {
                              if (var4) {
                                 var10000 = System.out;
                                 StringBuilder var31;
                                 var30 = var31 = new StringBuilder;
                                 var31.<init>("g has order phi(n)/");
                                 var10000.println(var30.append(var20.elementAt(var28)).append("\n g: ").append(var13).toString());
                              }

                              var33 = true;
                              break;
                           }
                        }

                        if (!var33) {
                           if (var13.modPow(var25.divide(BigInteger.valueOf(4L)), var17).equals(var14 = ONE)) {
                              if (var4) {
                                 System.out.println("g has order phi(n)/4\n g:" + var13);
                              }
                           } else if (var13.modPow(var25.divide(var15), var17).equals(var14)) {
                              if (var4) {
                                 System.out.println("g has order phi(n)/p'\n g: " + var13);
                              }
                           } else if (var13.modPow(var25.divide(var5), var17).equals(var14)) {
                              if (var4) {
                                 System.out.println("g has order phi(n)/q'\n g: " + var13);
                              }
                           } else if (var13.modPow(var25.divide(var9), var17).equals(var14)) {
                              if (var4) {
                                 System.out.println("g has order phi(n)/a\n g: " + var13);
                              }
                           } else {
                              if (!var13.modPow(var25.divide(var10), var17).equals(var14)) {
                                 if (var4) {
                                    var10000 = System.out;
                                    StringBuilder var21;
                                    var30 = var21 = new StringBuilder;
                                    var21.<init>("needed ");
                                    var10000.println(var30.append(var11).append(" tries to generate g").toString());
                                    System.out.println();
                                    System.out.println("found new NaccacheStern cipher variables:");
                                    System.out.println("smallPrimes: " + var20);
                                    System.out.println("sigma:...... " + var8 + " (" + var8.bitLength() + " bits)");
                                    System.out.println("a:.......... " + var9);
                                    System.out.println("b:.......... " + var10);
                                    System.out.println("p':......... " + var15);
                                    System.out.println("q':......... " + var5);
                                    System.out.println("p:.......... " + var16);
                                    System.out.println("q:.......... " + var7);
                                    System.out.println("n:.......... " + var17);
                                    System.out.println("phi(n):..... " + var25);
                                    System.out.println("g:.......... " + var13);
                                    System.out.println();
                                 }

                                 NaccacheSternKeyParameters var22;
                                 NaccacheSternKeyParameters var10002 = var22 = new NaccacheSternKeyParameters;
                                 int var23 = var8.bitLength();
                                 var10002.<init>(false, var13, var17, var23);
                                 NaccacheSternPrivateKeyParameters var24;
                                 NaccacheSternPrivateKeyParameters var32 = var24 = new NaccacheSternPrivateKeyParameters;
                                 var3 = var8.bitLength();
                                 var32.<init>(var13, var17, var3, var20, var25);
                                 return new AsymmetricCipherKeyPair(var22, var24);
                              }

                              if (var4) {
                                 System.out.println("g has order phi(n)/b\n g: " + var13);
                              }
                           }
                        }
                     }
                  }

                  if (var4) {
                     System.out.println("key size too small. Should be " + var1 + " but is actually " + var16.multiply(var7).bitLength());
                  }
               }
            }
         }
      }
   }
}
